/**
 * aiplayer.h
 *
 * A class for for an ai chess player
 *
 * Zach Kamsler
 * IMGD 4000
 */

#include "aiplayer.h"
#include "chessboard.h"
#include <algorithm>
#include <ClanLib/core.h>


int runAI(void * ai) {
	((AIPlayer*)ai)->go();
	return 0;
}

/**
 * Play
 */
bool AIPlayer::play(ChessBoard& board, Side side, Move& move) {
	if(!running) {
		b = &board;
		m = Move();
		done = false;
	
		start = CL_System::get_time();
		thread = CL_Thread(runAI, (void*)this);
		thread.start();
		running = true;

	} else if(done || CL_System::get_time()-start >= maxTime) {
		running = false;
		done = true;
		//thread.wait();

		if(m.null()) {
			return false;
		}

		//board.makeMove(m);
		move = m;
	
		return true;
	}

	return false;
}

/**
 * Does a minimax game tree search with alpha beta pruning
 *
 * @param depth		the depth to search to
 * @param alpha		the alpha cutoff
 * @param beta		the beta cutoff
 * @param board		the board
 * @param move		best move
 */
int AIPlayer::minimax(int depth, int alpha, int beta, const ChessBoard& board, Move& m) {
	if(done || depth <= 0) return board.evaluate();

	Move d;
	vector<Move> moves;
	board.generateMoves(moves);
	random_shuffle(moves.begin(), moves.end());

	if(moves.empty()) return board.evaluate();

	for(unsigned i = 0; i < moves.size(); i++) {
		ChessBoard newBoard(board);
		newBoard.makeMove(moves[i]);

		int val = -minimax(depth-1, -beta, -alpha, newBoard, d);

		if(val >= beta) return beta;
		if(val > alpha)  {
			alpha = val;
			m = moves[i];
		}
	}

	return alpha;
}

/**
 * Evaluates the given board
 *
 * @param board		the board to evaluate
 */
int AIPlayer::evaluate(const ChessBoard& board) {
	int material = 10;
	int safety = 25;
	int mobility = 1;

	// don't care about mobility early game, care lots endgame
	if(board.getNumMoves() < 10) {
		mobility = 0;
	} else if(board.getNumMoves() > 30) {
		mobility = 2;
		safety = 50;
	}

	// care more about pieces if aggressive
	if (aggressive) {
		material *= 2;
	}

	return board.evaluate(material, safety, mobility);

}

/**
 * Does an iterative deepening search for the best move
 */
void AIPlayer::go() {
	Move best;

	for(int depth = 1; !done && depth <= maxDepth; depth++) {
		minimax(depth, -INFINITY, INFINITY, *b, best);
		if (!done) m = best;
	}

	done = true;
}