#include "chessclient.h"
#include "chessboard.h"
#include "player.h"
#include <iostream>

void ChessClient::connect(const std::string& hostname) {
	CL_IPAddress address(hostname, PORT);
	cout << "connecting\n";
	sock.connect(address);
	cout << "connected\n";
	side = (Side)sock.input.read_char8();
	cout << "ok\n";
	valid = true;
}

void ChessClient::update() {
	if(!valid) return;

	cout << "updating\n";
	if(playing) {
		Move m;
		if(player.play(board, side, m)) {
			// send move to server
			sock.output.write_char8(m.start.row);
			sock.output.write_char8(m.start.col);
			sock.output.write_char8(m.end.row);
			sock.output.write_char8(m.end.col);

			// verify move
			if(sock.input.read_char8()) {
				board.makeMove(m);
			}

			playing = false;
		}
	} else {
		// read valid
		valid = sock.input.read_bool8();
		if(!valid) {
			cout << "Game over\n";
			return;
		}

		// read turn
		Side turn = (Side)sock.input.read_char8();

		if(turn == side) {
			playing = true;
		} else {
			// read move that was made
			char x1 = sock.input.read_char8();
			char y1 = sock.input.read_char8();
			char x2 = sock.input.read_char8();
			char y2 = sock.input.read_char8();

			// make move
			board.makeMove(Position(x1, y1), Position(x2, y2));
		}
	}
}

