/**
 * chessinterface.h
 *
 * Class representing the user interface for the chess game
 *
 * Zach Kamsler
 * IMGD 4000
 */

#ifndef _CHESSINTERFACE_H_
#define _CHESSINTERFACE_H_

#include <Clanlib/display.h>
#include "enums.h"
#include <vector>

class ChessBoard;
class CL_InputEvent;

/// Chess UI class
class ChessInterface {
public:
	ChessInterface(const ChessBoard& b);

	void draw();

	void setHighlighted(const std::vector<Position>& tiles);
	void clearHighlighted();
	void setCursor(const Position& pos);
	void hideCursor();

	CL_Signal_v1<const Position& > selectionChanged;
	void handleMouseClick(const CL_InputEvent& e);
private:
	const static int TILESIZE = 64;
	const ChessBoard& board;

	CL_Surface& getSurfaceFor(signed char piece);

	/// surfaces for drawing the chess board
	CL_Surface background;
	CL_Surface bottom;
	CL_Surface white[NUM_PIECES];
	CL_Surface black[NUM_PIECES];

	CL_Slot mousePress;

	Position selection;

	CL_Surface cursor;	///< the surface for the cursor
	bool showCursor; ///< whether to diplay the cursor
	Position cursorPos;	///< the position of the selection cursor

	CL_Font font; ///< the font to use

	CL_Surface highlight; ///< the surface for highlighted moves
	CL_Surface highlightAttack; ///< the surface for highlighted attack moves
	std::vector<Position> highlighted; ///< vector of the highlighted tiles
};

#endif

