/**
 * enums.h
 *
 * Basic enums and structures for the chess game
 *
 * Zach Kamsler
 * IMGD 4000
 */

#ifndef _ENUMS_H_
#define _ENUMS_H_

#include <climits>

/// enum for representing the side of a piece.
/// this is multiplied by the type to create the piece value.
enum Side {WHITE=1, BLACK=-1};

/// enum for representing the type of a piece
enum Piece {PAWN=1, KNIGHT, BISHOP, ROOK, QUEEN, KING, NUM_PIECES = KING};

/// struct for representing a position on the board
struct Position {
	Position() : row(0), col(0) {}
	Position(int r, int c) : row(r), col(c) {}

	Position operator+(const Position& o) const {
		return Position(row+o.row, col+o.col);
	}

	bool operator==(const Position& o) const {
		return row==o.row && col==o.col;
	}

	int row, col;
};

/// struct for reprenting a move
struct Move {
	Move() {}
	Move(const Position& s, const Position& e) : start(s), end(e) {}

	bool null() {Position p; return start==p && end == p;}
	
	Position start, end;
};

/// size of the board
const int BOARDSIZE = 8;
const int INFINITY = INT_MAX-1;

extern const char* const PORT;

bool sameSide(int a, int b);
Side reverseSide(Side a);

Side sideOf(signed char p);
bool isWhite(signed char p);
bool isBlack(signed char p);
bool hasType(signed char p, Piece t);

#endif

