/**
 * humanplayer.cpp
 *
 * A class for playing chess through the UI
 *
 * Zach Kamsler
 * IMGD 4000
 */

#include "humanplayer.h"
#include "chessinterface.h"
#include "chessboard.h"

/**
 * Begin the turn
 */
void HumanPlayer::beginTurn() {
	change = inter.selectionChanged.connect(this, HumanPlayer::selectionChanged);
	inTurn = true;
}

/**
 * End the turn
 */
void HumanPlayer::endTurn() {
	inter.selectionChanged.disconnect(change);
	inTurn = false;
	havePiece = false;
	haveDest = false;
	possible.clear();
	inter.hideCursor();
	inter.clearHighlighted();
}

/**
 * Play
 *
 * @param b		the board to play on
 * @param s		the side of the player
 *
 * @return true if the move was completed
 */
bool HumanPlayer::play(ChessBoard& b, Side s, Move& move) {
	board = &b;
	side = s;

	if(!inTurn) beginTurn();

	if(haveDest) {
		move = Move(piece, dest);
		endTurn();
		return true;
	}

	return false;
}


/**
 * React to change in interface selection
 *
 * @param pos	the tile selected
 */
void HumanPlayer::selectionChanged(const Position& pos) {
	// check if the selected tile has a piece on the players side
	if(board->hasSide(pos, side)) {
		// select the piece
		piece = pos;
		inter.setCursor(pos);

		// highlight possible moves
		possible.clear();
		board->movesFor(pos, possible);
		inter.setHighlighted(possible);
		havePiece = true;
	} 
	
	// if a piece is already selected
	else if (havePiece) {
		// check if the tile is a valid move
		bool moveOk = false;
		for(unsigned int i = 0; i < possible.size(); i++) {
			if(pos == possible[i]) {
				moveOk = true;
				break;
			}
		}

		// if the move is valid, make it
		if(moveOk) {
			dest = pos;
			haveDest = true;
			//board->makeMove(piece, dest);
		}

		inter.clearHighlighted();
		inter.hideCursor();
	}
}