/**
 * moves.h
 *
 * Classes for generating chess moves
 *
 * Zach Kamsler
 * IMGD 4000
 */

#ifndef _MOVES_H_
#define _MOVES_H_

#include <vector>
#include "enums.h"

class ChessBoard;
using namespace std;

/// base interface for moves
class Moves {
public:
	virtual void generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const = 0;
	virtual bool canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const = 0;
};

/// moves for knights
class KnightMoves : public Moves {
public:
	void generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const;
	bool canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const;
private:
	static const Position moveOffsets[];
	static const int numMoves;
};

/// moves for pawns
class PawnMoves : public Moves {
public:
	void generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const;
	bool canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const;
};

/// moves for pieces that move in certain directions
class DirectionalMoves : public Moves {
public:
	void generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const;
	bool canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const;
protected:
	DirectionalMoves() : distance(8) {}
	vector<Position> directions;
	int distance;
};

/// moves for kings
class KingMoves : public DirectionalMoves {
public:
	KingMoves();
};

/// moves for queens
class QueenMoves : public DirectionalMoves {
public:
	QueenMoves();
};

/// moves for rooks
class RookMoves : public DirectionalMoves {
public:
	RookMoves();
};

/// moves for bishops
class BishopMoves : public DirectionalMoves {
public:
	BishopMoves();
};

#endif

