/*
	Main server application
*/

#include <ClanLib/core.h>
#include <ClanLib/application.h>
#include <ClanLib/network.h>

#include "chessboard.h"

class ServerApplication : public CL_ClanApplication
{
public:
	virtual int main(int, char **)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Chess Server");
		console.redirect_stdio();

		// set up clanlib libraries
		CL_SetupCore setup_core;
		CL_SetupNetwork setup_network;

		// create sockets
		CL_Socket sock(CL_Socket::tcp);
		sock.bind(CL_IPAddress(PORT));
		sock.listen(2);

		while (1) {
			cout << "Starting new game\n";
			ChessBoard board;

			// connect to white player
			cout << "Connecting to white player... ";
			CL_Socket white = sock.accept();
			white.output.write_char8(WHITE);
			cout << "Connected\n";

			// connect to white player
			cout << "Connecting to black player... ";
			CL_Socket black = sock.accept();
			black.output.write_char8(BLACK);
			cout << "Connected\n";

			// start game
			while(!board.done()) {
				cout << "Making move...\n";

				// send turn info
				white.output.write_bool8(true);
				white.output.write_char8(board.getTurn());
				black.output.write_bool8(true);
				black.output.write_char8(board.getTurn());

				CL_Socket *current, *other;
				if(board.getTurn() == WHITE) {
					current = &white;
					other = &black;
				} else {
					current = &black;
					other = &white;
				}

				// get move
				char x1 = current->input.read_char8();
				char y1 = current->input.read_char8();
				char x2 = current->input.read_char8();
				char y2 = current->input.read_char8();

				cout << "Move recieved.\n";
				
				if(true) {
					board.makeMove(Position(x1, y1), Position(x2, y2));
					current->output.write_bool8(true);
					other->output.write_char8(x1);
					other->output.write_char8(y1);
					other->output.write_char8(x2);
					other->output.write_char8(y2);
				} else {
					current->output.write_bool8(false);
					other->output.write_char8(0);
					other->output.write_char8(0);
					other->output.write_char8(0);
					other->output.write_char8(0);
				}

			}

			cout << "Ending Game\n";
			white.output.write_bool8(false);
			black.output.write_bool8(false);
			cout << "Game ended\n";

		}


		return 0;
	}


} app;
